@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2019 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -34,10 +34,10 @@
 #include "utils.h"
 
 /**
- * Time interval in u-seconds to wait before enclosure management message is being
- * sent to AHCI controller.
+ * Time interval in nano seconds to wait before enclosure management message
+ * is being sent to AHCI controller.
  */
-#define EM_MSG_WAIT       1500
+#define EM_MSG_WAIT       1500000	/* 0.0015 seconds */
 
 /**
  * This array maps IBPI pattern to value recognized by AHCI driver. The driver
@@ -73,6 +73,10 @@ int ahci_sgpio_write(struct block_device *device, enum ibpi_pattern ibpi)
 	char temp[WRITE_BUFFER_SIZE];
 	char path[PATH_MAX];
 	char *sysfs_path = device->cntrl_path;
+	const struct timespec waittime = {
+		.tv_sec = 0,
+		.tv_nsec = EM_MSG_WAIT
+	};
 
 	/* write only if state has changed */
 	if (ibpi == device->ibpi_prev)
@@ -85,33 +89,41 @@ int ahci_sgpio_write(struct block_device *device, enum ibpi_pattern ibpi)
 
 	sprintf(temp, "%u", ibpi2sgpio[ibpi]);
 
-	str_cpy(path, sysfs_path, PATH_MAX);
-	str_cat(path, "/em_message", PATH_MAX);
+	snprintf(path, sizeof(path), "%s/em_message", sysfs_path);
 
-	usleep(EM_MSG_WAIT);
+	nanosleep(&waittime, NULL);
 	return buf_write(path, temp) > 0;
 }
 
+#define SCSI_HOST "/scsi_host"
 /*
  * The function return path to SATA port in sysfs tree. See ahci.h for details.
  */
+
 char *ahci_get_port_path(const char *path)
 {
-	char tmp[PATH_MAX], buf[BUFFER_MAX];
-	char *p, *s;
+	char *p;
+	char tmp[PATH_MAX];
+	char *buf;
+	size_t buf_size;
 
-	str_cpy(tmp, path, PATH_MAX);
-	p = strstr(tmp, "/target");
+	p = strstr(path, "/target");
 	if (p == NULL)
 		return NULL;
-	*p = '\0';
-	s = rindex(tmp, PATH_DELIM);
-	if (s == NULL)
+
+	if (sizeof(tmp) <= (p - path))
+		return NULL;
+	strncpy(tmp, path, p - path);
+	tmp[p - path] = '\0';
+	p = strrchr(tmp, PATH_DELIM);
+	if (p == NULL)
 		return NULL;
 
-	str_cpy(buf, s, BUFFER_MAX);
-	str_cat(tmp, "/scsi_host", PATH_MAX);
-	str_cat(tmp, buf, PATH_MAX);
+	buf_size = strlen(tmp) + strlen(p) + strlen(SCSI_HOST) + 1;
+	buf = malloc(buf_size);
+	if (buf == NULL)
+		return NULL;
 
-	return str_dup(tmp);
+	snprintf(buf, buf_size, "%s%s%s", tmp, SCSI_HOST, p);
+	return buf;
 }
