@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2020 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -36,6 +36,8 @@
 #include "status.h"
 #include "sysfs.h"
 #include "utils.h"
+#include "amd.h"
+#include "npem.h"
 
 /**
  * @brief Name of controllers types.
@@ -48,7 +50,9 @@ static const char * const ctrl_type_str[] = {
 	[CNTRL_TYPE_DELLSSD] = "Dell SSD",
 	[CNTRL_TYPE_VMD]     = "VMD",
 	[CNTRL_TYPE_SCSI]    = "SCSI",
-	[CNTRL_TYPE_AHCI]    = "AHCI"
+	[CNTRL_TYPE_AHCI]    = "AHCI",
+	[CNTRL_TYPE_NPEM]    = "NPEM",
+	[CNTRL_TYPE_AMD]     = "AMD",
 };
 
 /**
@@ -68,33 +72,93 @@ static int _is_isci_cntrl(const char *path)
 
 /**
  */
-static int _is_ahci_cntrl(const char *path)
+static int _is_cntrl(const char *path, const char *type)
 {
 	char temp[PATH_MAX], link[PATH_MAX], *t;
 
-	str_cpy(temp, path, PATH_MAX);
-	str_cat(temp, "/driver", PATH_MAX);
+	snprintf(temp, sizeof(temp), "%s/driver", path);
 
 	if (realpath(temp, link) == NULL)
 		return 0;
 
 	t = strrchr(link, '/');
-	if ((t != NULL) && (strcmp(t + 1, "ahci") != 0))
+	if ((t != NULL) && (strcmp(t + 1, type) != 0))
+		return 0;
+
+	return 1;
+}
+
+static int _is_ahci_cntrl(const char *path)
+{
+	return _is_cntrl(path, "ahci");
+}
+
+static int _is_nvme_cntrl(const char *path)
+{
+	return _is_cntrl(path, "nvme");
+}
+
+static int _is_intel_ahci_cntrl(const char *path)
+{
+	if (!_is_ahci_cntrl(path))
 		return 0;
 
 	return get_uint64(path, 0, "vendor") == 0x8086L;
 }
 
+static int _is_amd_ahci_cntrl(const char *path)
+{
+	if (!_is_ahci_cntrl(path))
+		return 0;
+
+	return get_uint64(path, 0, "vendor") == 0x1022L;
+}
+
+static int _is_amd_nvme_cntrl(const char *path)
+{
+	char tmp[PATH_MAX];
+	char *t;
+
+	if (!_is_nvme_cntrl(path))
+		return 0;
+
+	sprintf(tmp, "%s", path);
+	t = strrchr(tmp, '/');
+	if (!t)
+		return 0;
+
+	t++;
+	*t = '\0';
+	return get_uint64(tmp, 0, "vendor") == 0x1022L;
+}
+
+static int _is_amd_cntrl(const char *path)
+{
+	if (_is_amd_ahci_cntrl(path))
+		return 1;
+
+	if (_is_amd_nvme_cntrl(path))
+		return 1;
+
+	return 0;
+}
+
+extern int get_dell_server_type(void);
+
 static int _is_dellssd_cntrl(const char *path)
 {
 	uint64_t vdr, dev, svdr, cls;
+	int gen = 0;
 
 	vdr = get_uint64(path, 0, "vendor");
 	dev = get_uint64(path, 0, "device");
 	cls = get_uint64(path, 0, "class");
 	svdr = get_uint64(path, 0, "subsystem_vendor");
+	if (cls == 0x10802)
+		gen = get_dell_server_type();
 
 	return ((vdr == 0x1344L && dev == 0x5150L) || /* micron ssd */
+		(gen != 0) ||			      /* Dell Server+NVME */
 	        (svdr == 0x1028 && cls == 0x10802));  /* nvmhci ssd */
 }
 
@@ -135,6 +199,11 @@ static int _is_vmd_cntrl(const char *path)
 	return sysfs_check_driver(path, "vmd");
 }
 
+static int _is_npem_cntrl(const char *path)
+{
+	return is_npem_capable(path);
+}
+
 /**
  * @brief Determines the type of controller.
  *
@@ -151,14 +220,17 @@ static int _is_vmd_cntrl(const char *path)
 static enum cntrl_type _get_type(const char *path)
 {
 	enum cntrl_type type = CNTRL_TYPE_UNKNOWN;
-
-	if (_is_vmd_cntrl(path)) {
+	if (_is_npem_cntrl(path)) {
+		type = CNTRL_TYPE_NPEM;
+	} else if (_is_vmd_cntrl(path)) {
 		type = CNTRL_TYPE_VMD;
 	} else if (_is_dellssd_cntrl(path)) {
 		type = CNTRL_TYPE_DELLSSD;
 	} else if (_is_storage_controller(path)) {
-		if (_is_ahci_cntrl(path))
+		if (_is_intel_ahci_cntrl(path))
 			type = CNTRL_TYPE_AHCI;
+		else if (_is_amd_cntrl(path))
+			type = CNTRL_TYPE_AMD;
 		else if (_is_isci_cntrl(path)
 				|| sysfs_enclosure_attached_to_cntrl(path)
 				|| _is_smp_cntrl(path))
@@ -327,11 +399,15 @@ struct cntrl_device *cntrl_device_init(const char *path)
 		case CNTRL_TYPE_DELLSSD:
 		case CNTRL_TYPE_SCSI:
 		case CNTRL_TYPE_VMD:
+		case CNTRL_TYPE_NPEM:
 			em_enabled = 1;
 			break;
 		case CNTRL_TYPE_AHCI:
 			em_enabled = _ahci_em_messages(path);
 			break;
+		case CNTRL_TYPE_AMD:
+			em_enabled = amd_em_enabled(path);
+			break;
 		default:
 			em_enabled = 0;
 		}
@@ -369,7 +445,7 @@ struct cntrl_device *cntrl_device_init(const char *path)
 					device->hosts = NULL;
 				}
 				device->cntrl_type = type;
-				device->sysfs_path = strdup(path);
+				device->sysfs_path = str_dup(path);
 			}
 		} else {
 			log_error
