@@ -1,7 +1,7 @@
 /*
  * Intel(R) Enclosure LED Utilities
  *
- * Copyright (C) 2017-2018 Intel Corporation.
+ * Copyright (C) 2017-2019 Intel Corporation.
  * Copyright (C) 2009 Karel Zak <kzak@redhat.com>
  *
  * SPDX-License-Identifier: GPL-2.0
@@ -15,6 +15,7 @@
 
 #include "list.h"
 
+#define LEDMON_SHARE_MEM_FILE "/ledmon.conf"
 #define LEDMON_DEF_CONF_FILE "/etc/ledmon.conf"
 #define LEDMON_DEF_LOG_FILE "/var/log/ledmon.log"
 #define LEDCTL_DEF_LOG_FILE "/var/log/ledctl.log"
@@ -41,7 +42,7 @@ struct ledmon_conf {
 	int blink_on_migration;
 	int blink_on_init;
 	int rebuild_blink_on_all;
-	int raid_memebers_only;
+	int raid_members_only;
 
 	/* whitelist and blacklist of controllers for blinking */
 	struct list cntrls_whitelist;
@@ -51,5 +52,7 @@ struct ledmon_conf {
 extern struct ledmon_conf conf;
 
 int ledmon_read_config(const char *filename);
+int ledmon_write_shared_conf(void);
+int ledmon_remove_shared_conf(void);
 
 #endif /* SRC_CONFIG_FILE_H_ */
