@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2019 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -95,38 +95,34 @@ const char *ibpi_str[] = {
  * Internal variable of ledctl utility. It is the pattern used to print out
  * information about the version of ledctl utility.
  */
-static char *ledctl_version = "Intel(R) Enclosure LED Control Application %d.%d\n"
-			      "Copyright (C) 2009-2018 Intel Corporation.\n";
+static char *ledctl_version = "Intel(R) Enclosure LED Control Application %d.%d %s\n"
+			      "Copyright (C) 2009-2019 Intel Corporation.\n";
 
 /**
  * Internal variable of monitor service. It is used to help parse command line
  * short options.
  */
-static char *shortopt = "hLxvl:";
+static char *shortopt;
 
-/**
- * Internal enumeration type. It is used to help parse command line arguments.
- */
-enum longopt {
+struct option *longopt;
+
+static int possible_params[] = {
 	OPT_HELP,
 	OPT_LOG,
 	OPT_VERSION,
 	OPT_LIST_CTRL,
 	OPT_LISTED_ONLY,
+	OPT_ALL,
+	OPT_DEBUG,
+	OPT_ERROR,
+	OPT_INFO,
+	OPT_QUIET,
+	OPT_WARNING,
+	OPT_LOG_LEVEL,
 };
 
-/**
- * Internal array with option tokens. It is used to help parse command line
- * long options.
- */
-static struct option longopt[] = {
-	[OPT_HELP]    = {"help", no_argument, NULL, 'h'},
-	[OPT_LOG]     = {"log", required_argument, NULL, 'l'},
-	[OPT_VERSION] = {"version", no_argument, NULL, 'v'},
-	[OPT_LIST_CTRL] = {"list-controllers", no_argument, NULL, 'L'},
-	[OPT_LISTED_ONLY] = {"listed-only", no_argument, NULL, 'x'},
-			{NULL, no_argument, NULL, '\0'}
-};
+static const int possible_params_size = sizeof(possible_params)
+		/ sizeof(possible_params[0]);
 
 static int listed_only;
 
@@ -148,8 +144,8 @@ static void ibpi_state_fini(struct ibpi_state *p)
  *
  * @return The function does not return a value.
  */
-static void _ledctl_fini(int __attribute__ ((unused)) status,
-			 void *__attribute__ ((unused)) ignore)
+static void _ledctl_fini(int status __attribute__ ((unused)),
+			 void *ignore __attribute__ ((unused)))
 {
 	sysfs_reset();
 	list_erase(&ibpi_list);
@@ -167,7 +163,7 @@ static void _ledctl_fini(int __attribute__ ((unused)) status,
  */
 static void _ledctl_version(void)
 {
-	printf(ledctl_version, VERSION_MAJOR, VERSION_MINOR);
+	printf(ledctl_version, VERSION_MAJOR, VERSION_MINOR, BUILD_LABEL);
 	printf("\nThis is free software; see the source for copying conditions." \
 	       " There is NO warranty;\nnot even for MERCHANTABILITY or FITNESS" \
 	       " FOR A PARTICULAR PURPOSE.\n\n");
@@ -185,7 +181,7 @@ static void _ledctl_version(void)
  */
 static void _ledctl_help(void)
 {
-	printf(ledctl_version, VERSION_MAJOR, VERSION_MINOR);
+	printf(ledctl_version, VERSION_MAJOR, VERSION_MINOR, BUILD_LABEL);
 	printf("\nUsage: %s [OPTIONS] pattern=list_of_devices ...\n\n",
 	       progname);
 	printf("Mandatory arguments for long options are mandatory for short options, too.\n\n");
@@ -198,6 +194,8 @@ static void _ledctl_help(void)
 	print_opt("--help", "-h", "Displays this help text.");
 	print_opt("--version", "-v",
 			  "Displays version and license information.");
+	print_opt("--log-level=VALUE", "-l VALUE",
+			  "Allows user to set ledctl verbose level in logs.");
 	printf("\nPatterns:\n"
 	       "\tCommon patterns are:\n"
 	       "\t\tlocate, locate_off, normal, off, degraded, rebuild,\n" ""
@@ -269,7 +267,7 @@ static void _determine(struct ibpi_state *state)
 	} else {
 		log_warning
 		    ("IBPI %s: missing block device(s)... pattern ignored.",
-		     ibpi_str[state->ibpi]);
+		     ibpi2str(state->ibpi));
 	}
 }
 
@@ -280,19 +278,19 @@ static void _determine(struct ibpi_state *state)
  * of IBPI states and calls _determine() function for each element. If the list
  * is empty the function logs a warning message and does nothing.
  *
- * @param[in]      ibpi_list      pointer to list of IBPI states.
+ * @param[in]      ibpi_local_list  pointer to list of IBPI states.
  *
  * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
  *         The following status codes function returns:
  *
  *         STATUS_LIST_EMPTY      the specified list has no elements.
  */
-static status_t _ibpi_state_determine(struct list *ibpi_list)
+static status_t _ibpi_state_determine(struct list *ibpi_local_list)
 {
-	if (list_is_empty(ibpi_list) == 0) {
+	if (list_is_empty(ibpi_local_list) == 0) {
 		struct ibpi_state *state;
 
-		list_for_each(ibpi_list, state)
+		list_for_each(ibpi_local_list, state)
 			_determine(state);
 		return STATUS_SUCCESS;
 	}
@@ -300,59 +298,18 @@ static status_t _ibpi_state_determine(struct list *ibpi_list)
 	return STATUS_LIST_EMPTY;
 }
 
-static struct ibpi_state *_ibpi_find(const struct list *ibpi_list,
+static struct ibpi_state *_ibpi_find(const struct list *ibpi_local_list,
 				     enum ibpi_pattern ibpi)
 {
 	struct ibpi_state *state;
 
-	list_for_each(ibpi_list, state) {
+	list_for_each(ibpi_local_list, state) {
 		if (state->ibpi == ibpi)
 			return state;
 	}
 	return NULL;
 }
 
-/**
- * @brief Sets the path to configuration file.
- *
- * This is internal function of monitor service. This function sets the path and
- * name of configuration file. The function is checking whether the given path
- * is valid or it is invalid and should be ignored.
- *
- * @param[in]      path           the new location and name of config file.
- *
- * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
- */
-static status_t _set_config_path(const char *path)
-{
-	(void)path;
-	return STATUS_SUCCESS;
-}
-
-/**
- * @brief Sets the path to local log file.
- *
- * This is internal function of ledctl utility. This function sets the path and
- * file name of log file. The function checks if the specified path is valid. If
- * the path is invalid then the function does nothing.
- *
- * @param[in]      path           new location and name of log file.
- *
- * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
- */
-static status_t _set_log_path(const char *path)
-{
-	char temp[PATH_MAX];
-
-	if (realpath(path, temp) == NULL) {
-		if ((errno != ENOENT) && (errno != ENOTDIR))
-			return STATUS_INVALID_PATH;
-	}
-	if (log_open(temp) < 0)
-		return STATUS_FILE_OPEN_ERROR;
-	return STATUS_SUCCESS;
-}
-
 /**
  * @brief Gets a pointer to IBPI state structure.
  *
@@ -484,7 +441,7 @@ static status_t _ibpi_state_add_block(struct ibpi_state *state, char *name)
 	if (strstr(temp, "/dev/") != NULL) {
 		if (stat(temp, &st) < 0)
 			return STATUS_STAT_ERROR;
-		sprintf(temp, "/sys/dev/block/%d:%d", major(st.st_rdev),
+		sprintf(temp, "/sys/dev/block/%u:%u", major(st.st_rdev),
 			minor(st.st_rdev));
 		if ((realpath(temp, path) == NULL) && (errno != ENOTDIR))
 			return STATUS_INVALID_PATH;
@@ -501,7 +458,7 @@ static status_t _ibpi_state_add_block(struct ibpi_state *state, char *name)
 		list_append(&state->block_list, blk1);
 	else
 		log_info("%s: %s: device already on the list.",
-			 ibpi_str[state->ibpi], path);
+			 ibpi2str(state->ibpi), path);
 	return STATUS_SUCCESS;
 }
 
@@ -522,6 +479,8 @@ static status_t _ibpi_state_add_block(struct ibpi_state *state, char *name)
  */
 static status_t _cmdline_ibpi_parse(int argc, char *argv[])
 {
+	status_t t_status, ret_status = STATUS_SUCCESS;
+
 	while (optind < argc) {
 		struct ibpi_state *state = NULL;
 		char *p = argv[optind++];
@@ -530,29 +489,35 @@ static status_t _cmdline_ibpi_parse(int argc, char *argv[])
 		if (*t != '\0') {
 			*(t++) = '\0';
 			state = _ibpi_state_get(p);
-			if (state != NULL) {
-				if (*t == '{') {
-					while ((t = argv[optind++]) != NULL) {
-						if (*t == '}')
-							break;
-						_ibpi_state_add_block(state, t);
-					}
-				} else {
-					while (*(p = t) != '\0') {
-						t = strchrnul(p, ',');
-						if (*t != '\0')
-							*(t++) = '\0';
-						_ibpi_state_add_block(state, p);
-					}
+			if (state == NULL) {
+				log_error("%s - unknown pattern name.", p);
+				return STATUS_INVALID_STATE;
+			}
+			if (*t == '{') {
+				while ((t = argv[optind++]) != NULL) {
+					if (*t == '}')
+						break;
+					t_status =
+					_ibpi_state_add_block(state, t);
+					if (t_status != STATUS_SUCCESS)
+						ret_status = t_status;
+				}
+			} else {
+				while (*(p = t) != '\0') {
+					t = strchrnul(p, ',');
+					if (*t != '\0')
+						*(t++) = '\0';
+					t_status =
+					_ibpi_state_add_block(state, p);
+					if (t_status != STATUS_SUCCESS)
+						ret_status = t_status;
 				}
 			}
 		}
-		if (state == NULL) {
-			log_error("%s - unknown pattern name.", p);
-			exit(STATUS_INVALID_STATE);
-		}
 	}
-	return STATUS_SUCCESS;
+	if (_ibpi_state_determine(&ibpi_list) != STATUS_SUCCESS)
+		ret_status = STATUS_IBPI_DETERMINE_ERROR;
+	return ret_status;
 }
 
 /**
@@ -577,17 +542,31 @@ static status_t _cmdline_parse(int argc, char *argv[])
 		if (opt == -1)
 			break;
 		switch (opt) {
+		int log_level;
+
+		case 0:
+			switch (get_option_id(longopt[opt_index].name)) {
+			case OPT_LOG_LEVEL:
+				log_level = get_option_id(optarg);
+				if (log_level != -1)
+					status = set_verbose_level(log_level);
+				else
+					status = STATUS_CMDLINE_ERROR;
+				break;
+			default:
+				status = set_verbose_level(
+						possible_params[opt_index]);
+
+			}
+			break;
 		case 'v':
 			_ledctl_version();
 			exit(EXIT_SUCCESS);
 		case 'h':
 			_ledctl_help();
 			exit(EXIT_SUCCESS);
-		case 'c':
-			status = _set_config_path(optarg);
-			break;
 		case 'l':
-			status = _set_log_path(optarg);
+			status = set_log_path(optarg);
 			break;
 		case 'x':
 			status = STATUS_SUCCESS;
@@ -628,24 +607,21 @@ static status_t _cmdline_parse(int argc, char *argv[])
  * @param[in]      sysfs          pointer to sysfs structure holding information
  *                                about the existing controllers, block devices,
  *                                and software RAID devices.
- * @param[in]      ibpi_list      TBD
+ * @param[in]      ibpi_local_list  TBD
  *
  * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
  */
-static status_t _ledctl_execute(struct list *ibpi_list)
+static status_t _ledctl_execute(struct list *ibpi_local_list)
 {
 	struct ibpi_state *state;
 	struct block_device *device;
 
-	if (_ibpi_state_determine(ibpi_list) != STATUS_SUCCESS)
-		return STATUS_IBPI_DETERMINE_ERROR;
-
 	if (!listed_only) {
 		list_for_each(sysfs_get_block_devices(), device)
 			device->send_fn(device, IBPI_PATTERN_LOCATE_OFF);
 	}
 
-	list_for_each(ibpi_list, state)
+	list_for_each(ibpi_local_list, state)
 		list_for_each(&state->block_list, device)
 			device->send_fn(device, device->ibpi);
 
@@ -655,15 +631,28 @@ static status_t _ledctl_execute(struct list *ibpi_list)
 	return STATUS_SUCCESS;
 }
 
+static status_t _read_shared_conf(void)
+{
+	status_t status;
+	char share_conf_path[PATH_MAX];
+
+	memset(share_conf_path, 0, sizeof(share_conf_path));
+	snprintf(share_conf_path, sizeof(share_conf_path), "/dev/shm%s",
+		 LEDMON_SHARE_MEM_FILE);
+
+	status = ledmon_read_config(share_conf_path);
+	return status;
+}
+
 static status_t _init_ledctl_conf(void)
 {
 	memset(&conf, 0, sizeof(struct ledmon_conf));
-
 	/* initialize with default values */
 	conf.log_level = LOG_LEVEL_WARNING;
 	list_init(&conf.cntrls_whitelist, NULL);
 	list_init(&conf.cntrls_blacklist, NULL);
-	return _set_log_path(LEDCTL_DEF_LOG_FILE);
+
+	return set_log_path(LEDCTL_DEF_LOG_FILE);
 }
 
 /**
@@ -687,6 +676,8 @@ int main(int argc, char *argv[])
 {
 	status_t status;
 
+	setup_options(&longopt, &shortopt, possible_params,
+			possible_params_size);
 	set_invocation_name(argv[0]);
 	openlog(progname, LOG_PERROR, LOG_USER);
 
@@ -702,6 +693,15 @@ int main(int argc, char *argv[])
 		exit(STATUS_ONEXIT_ERROR);
 	if (_cmdline_parse(argc, argv))
 		exit(STATUS_CMDLINE_ERROR);
+	free(shortopt);
+	free(longopt);
+	status = _read_shared_conf();
+	if (status != STATUS_SUCCESS)
+		return status;
+	status = log_open(conf.log_path);
+	if (status != STATUS_SUCCESS)
+		return STATUS_LOG_FILE_ERROR;
+
 	list_init(&ibpi_list, (item_free_t)ibpi_state_fini);
 	sysfs_init();
 	sysfs_scan();
@@ -709,7 +709,7 @@ int main(int argc, char *argv[])
 	if (status != STATUS_SUCCESS) {
 		log_debug("main(): _ibpi_parse() failed (status=%s).",
 			  strstatus(status));
-		exit(STATUS_INVALID_STATE);
+		exit(status);
 	}
 	return _ledctl_execute(&ibpi_list);
 }
