@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2019 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -21,6 +21,8 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <inttypes.h>
+#include <libgen.h>
 #include <limits.h>
 #include <regex.h>
 #include <stdarg.h>
@@ -30,7 +32,6 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -39,18 +40,10 @@
 #endif
 
 #include "config.h"
-#include "config_file.h"
 #include "list.h"
 #include "status.h"
 #include "utils.h"
 
-/**
- */
-#define PREFIX_DEBUG          "  DEBUG: "
-#define PREFIX_WARNING        "WARNING: "
-#define PREFIX_INFO           "   INFO: "
-#define PREFIX_ERROR          "  ERROR: "
-
 /**
  */
 #define TIMESTAMP_PATTERN    "%b %d %T "
@@ -64,6 +57,13 @@ char *progname = NULL;
  */
 static FILE *s_log = NULL;
 
+struct log_level_info log_level_infos[] = {
+		[LOG_LEVEL_DEBUG] = {PREFIX_DEBUG, LOG_DEBUG},
+		[LOG_LEVEL_WARNING] = {PREFIX_WARNING, LOG_WARNING},
+		[LOG_LEVEL_INFO] = {PREFIX_INFO, LOG_INFO},
+		[LOG_LEVEL_ERROR] = {PREFIX_ERROR, LOG_ERR}
+};
+
 /*
  * Function returns a content of a text file. See utils.h for details.
  */
@@ -71,10 +71,7 @@ char *get_text(const char *path, const char *name)
 {
 	char temp[PATH_MAX];
 
-	str_cpy(temp, path, PATH_MAX);
-	str_cat(temp, PATH_DELIM_STR, PATH_MAX);
-	str_cat(temp, name, PATH_MAX);
-
+	snprintf(temp, sizeof(temp), "%s/%s", path, name);
 	return buf_read(temp);
 }
 
@@ -102,11 +99,14 @@ int get_bool(const char *path, int defval, const char *name)
 uint64_t get_uint64(const char *path, uint64_t defval, const char *name)
 {
 	char *p = get_text(path, name);
+	uint64_t retval = defval;
+
 	if (p) {
-		sscanf(p, "0x%Lx", (long long unsigned int *)&defval);
+		if (sscanf(p, "%" SCNx64, &defval) == 1)
+			retval = defval;
 		free(p);
 	}
-	return defval;
+	return retval;
 }
 
 /*
@@ -174,9 +174,10 @@ static int _is_virtual(int dev_type)
 
 /**
  */
-int buf_write(const char *path, const char *buf)
+ssize_t buf_write(const char *path, const char *buf)
 {
-	int fd, size = -1;
+	int fd;
+	ssize_t size = -1;
 
 	if (path == NULL)
 		__set_errno_and_return(EINVAL);
@@ -259,45 +260,17 @@ static void _log_timestamp(void)
 	}
 }
 
-/**
- */
-static int _mkdir(const char *path)
-{
-	char temp[PATH_MAX];
-	int status = -1;
-	char *t = realpath(path, temp);
-	while (t) {
-		t = strchr(t + 1, PATH_DELIM);
-		if (t)
-			*t = '\0';
-		status = mkdir(temp, 0640);
-		if (t)
-			*t = PATH_DELIM;
-		if ((status < 0) && (errno != EEXIST))
-			break;
-		status = 0;
-	}
-	return status;
-}
-
 /**
  */
 int log_open(const char *path)
 {
 	if (s_log)
 		log_close();
-	char *t = rindex(path, PATH_DELIM);
-	if (t)
-		*t = '\0';
-	int status = _mkdir(path);
-	if (t)
-		*t = PATH_DELIM;
-	if (status == 0) {
-		s_log = fopen(path, "a");
-		if (s_log == NULL)
-			return -1;
-	}
-	return status;
+
+	s_log = fopen(path, "a");
+	if (s_log == NULL)
+		return -1;
+	return 0;
 }
 
 /**
@@ -309,94 +282,30 @@ void log_close(void)
 		fclose(s_log);
 		s_log = NULL;
 	}
-	closelog();
 }
 
 /**
  */
-void log_debug(const char *buf, ...)
+void _log(enum log_level_enum loglevel, const char *buf,  ...)
 {
 	va_list vl;
+	struct log_level_info *lli = &log_level_infos[loglevel];
 
 	if (s_log == NULL)
 		log_open(conf.log_path);
-	if (s_log && (conf.log_level >= LOG_LEVEL_DEBUG)) {
-		_log_timestamp();
-		fprintf(s_log, PREFIX_DEBUG);
-		va_start(vl, buf);
-		vfprintf(s_log, buf, vl);
-		va_end(vl);
-		fprintf(s_log, "\n");
-		fflush(s_log);
-		va_start(vl, buf);
-		vsyslog(LOG_DEBUG, buf, vl);
-		va_end(vl);
-	}
-}
 
-/**
- */
-void log_error(const char *buf, ...)
-{
-	va_list vl;
-
-	if (s_log == NULL)
-		log_open(conf.log_path);
-	if (s_log && (conf.log_level >= LOG_LEVEL_ERROR)) {
-		_log_timestamp();
-		fprintf(s_log, PREFIX_ERROR);
+	if (conf.log_level >= loglevel) {
+		char msg[4096];
 		va_start(vl, buf);
-		vfprintf(s_log, buf, vl);
-		va_end(vl);
-		fprintf(s_log, END_LINE_STR);
-		fflush(s_log);
-		va_start(vl, buf);
-		vsyslog(LOG_ERR, buf, vl);
-		va_end(vl);
-	}
-}
-
-/**
- */
-void log_warning(const char *buf, ...)
-{
-	va_list vl;
-
-	if (s_log == NULL)
-		log_open(conf.log_path);
-	if (s_log && (conf.log_level >= LOG_LEVEL_WARNING)) {
-		_log_timestamp();
-		fprintf(s_log, PREFIX_WARNING);
-		va_start(vl, buf);
-		vfprintf(s_log, buf, vl);
-		va_end(vl);
-		fprintf(s_log, END_LINE_STR);
-		fflush(s_log);
-		va_start(vl, buf);
-		vsyslog(LOG_WARNING, buf, vl);
-		va_end(vl);
-	}
-}
-
-/**
- */
-void log_info(const char *buf, ...)
-{
-	va_list vl;
-
-	if (s_log == NULL)
-		log_open(conf.log_path);
-	if (s_log && (conf.log_level >= LOG_LEVEL_INFO)) {
-		_log_timestamp();
-		fprintf(s_log, PREFIX_INFO);
-		va_start(vl, buf);
-		vfprintf(s_log, buf, vl);
-		va_end(vl);
-		fprintf(s_log, END_LINE_STR);
-		fflush(s_log);
-		va_start(vl, buf);
-		vsyslog(LOG_INFO, buf, vl);
+		vsnprintf(msg, sizeof(msg), buf, vl);
 		va_end(vl);
+		if (s_log) {
+			_log_timestamp();
+			fprintf(s_log, "%s", lli->prefix);
+			fprintf(s_log, "%s\n", msg);
+			fflush(s_log);
+		}
+		syslog(lli->priority, "%s", msg);
 	}
 }
 
@@ -417,7 +326,7 @@ void set_invocation_name(char *invocation_name)
 	(void)invocation_name;
 	progname = program_invocation_short_name;
 #else
-	char *t = rindex(invocation_name, PATH_DELIM);
+	char *t = strrchr(invocation_name, PATH_DELIM);
 	if (t)
 		progname = t + 1;
 	else
@@ -438,25 +347,21 @@ char *str_cpy(char *dest, const char *src, size_t size)
  */
 char *str_dup(const char *src)
 {
-	if (src && (strlen(src) > 0))
-		return strdup(src);
-	return NULL;
-}
+	char *ret;
 
-/**
- */
-char *str_cat(char *dest, const char *src, size_t size)
-{
-	int t = strlen(dest);
-	strncat(dest, src, size - t);
-	if (t + strlen(src) >= size)
-		dest[size - 1] = '\0';
-	return dest;
+	if (!src)
+		return NULL;
+	ret = strdup(src);
+	if (!ret) {
+		log_error("Cannot duplicate string");
+		exit(EXIT_FAILURE);
+	}
+	return ret;
 }
 
 char *get_path_hostN(const char *path)
 {
-	char *c = NULL, *s = NULL, *p = strdup(path);
+	char *c = NULL, *s = NULL, *p = str_dup(path);
 	if (!p)
 		return NULL;
 	c = strstr(p, "host");
@@ -466,7 +371,7 @@ char *get_path_hostN(const char *path)
 	if (!s)
 		goto end;
 	*s = 0;
-	s = strdup(c);
+	s = str_dup(c);
  end:
 	free(p);
 	return s;
@@ -475,7 +380,7 @@ char *get_path_hostN(const char *path)
 char *get_path_component_rev(const char *path, int index)
 {
 	int i;
-	char *c = NULL, *p = strdup(path);
+	char *c = NULL, *p = str_dup(path);
 	char *result = NULL;
 	for (i = 0; i <= index; i++) {
 		if (c)
@@ -483,7 +388,7 @@ char *get_path_component_rev(const char *path, int index)
 		c = strrchr(p, '/');
 	}
 	if (c)
-		result = strdup(c + 1);
+		result = str_dup(c + 1);
 	free(p);
 	return result;
 }
@@ -500,7 +405,7 @@ char *truncate_path_component_rev(const char *path, int index)
 			*c = '\0';
 		c = strrchr(p, '/');
 	}
-	c = strdup(p);
+	c = str_dup(p);
 	free(p);
 	return c;
 }
@@ -518,7 +423,7 @@ int match_string(const char *string, const char *pattern)
 
 	status = regcomp(&regex, pattern, REG_EXTENDED);
 	if (status != 0) {
-		log_debug("regecomp failed, ret=%d", __FUNCTION__, status);
+		log_debug("regecomp failed, ret=%d", status);
 		return 0;
 	}
 
@@ -531,7 +436,9 @@ int match_string(const char *string, const char *pattern)
 
 int get_log_fd(void)
 {
-	return fileno(s_log);
+	if (s_log)
+		return fileno(s_log);
+	return -1;
 }
 
 void print_opt(const char *long_opt, const char *short_opt, const char *desc)
@@ -558,17 +465,179 @@ void print_opt(const char *long_opt, const char *short_opt, const char *desc)
 status_t set_log_path(const char *path)
 {
 	char temp[PATH_MAX];
+	char log_file[PATH_MAX];
+	char *resolved, *logdir, *logfile, *cpath;
 
-	if (realpath(path, temp) == NULL) {
-		if ((errno != ENOENT) && (errno != ENOTDIR))
-			return STATUS_INVALID_PATH;
+	/*
+	 * Extract directory from path
+	 */
+	cpath = str_dup(path);
+	logdir = dirname(cpath);
+
+	/*
+	 * Check if directory exists
+	 */
+	resolved = realpath(logdir, temp);
+	if (resolved == NULL) {
+		printf("%s: %s\n", strerror(errno), logdir);
+		free(cpath);
+		return STATUS_INVALID_PATH;
 	}
-	if (log_open(temp) < 0)
-		return STATUS_FILE_OPEN_ERROR;
+
+	free(cpath);
+	cpath = str_dup(path);
+	logfile = basename(cpath);
+
+	snprintf(log_file, sizeof(log_file), "%s/%s",
+		 resolved, logfile);
+	free(cpath);
 
 	if (conf.log_path)
 		free(conf.log_path);
-	conf.log_path = strdup(temp);
+	conf.log_path = str_dup(log_file);
 
 	return STATUS_SUCCESS;
 }
+
+/**
+ * Internal array with option tokens. It is used to help parse command line
+ * long options.
+ */
+struct option longopt_all[] = {
+	[OPT_ALL]          = {"all", no_argument, NULL, '\0'},
+	[OPT_CONFIG]       = {"config", required_argument, NULL, 'c'},
+	[OPT_DEBUG]        = {"debug", no_argument, NULL, '\0'},
+	[OPT_ERROR]        = {"error", no_argument, NULL, '\0'},
+	[OPT_HELP]         = {"help", no_argument, NULL, 'h'},
+	[OPT_INFO]         = {"info", no_argument, NULL, '\0'},
+	[OPT_INTERVAL]     = {"interval", required_argument, NULL, 't'},
+	[OPT_LOG]          = {"log", required_argument, NULL, 'l'},
+	[OPT_QUIET]        = {"quiet", no_argument, NULL, '\0'},
+	[OPT_VERSION]      = {"version", no_argument, NULL, 'v'},
+	[OPT_WARNING]      = {"warning", no_argument, NULL, '\0'},
+	[OPT_LOG_LEVEL]    = {"log-level", required_argument, NULL, '\0'},
+	[OPT_LIST_CTRL]    = {"list-controllers", no_argument, NULL, 'L'},
+	[OPT_LISTED_ONLY]  = {"listed-only", no_argument, NULL, 'x'},
+	[OPT_FOREGROUND]   = {"foreground", no_argument, NULL, '\0'},
+	[OPT_NULL_ELEMENT] = {NULL, no_argument, NULL, '\0'}
+};
+
+void setup_options(struct option **_longopt, char **_shortopt, int *options, int
+		options_nr)
+{
+	struct option *longopt;
+	char *shortopt;
+	int i, j = 0;
+	struct option *opt;
+
+	longopt = malloc(sizeof(struct option) * (options_nr + 1));
+	shortopt = calloc(options_nr * 2 + 1, sizeof(char));
+	if (!longopt || !shortopt) {
+		fprintf(stderr, "Out of memory\n");
+		exit(STATUS_OUT_OF_MEMORY);
+	}
+	for (i = 0; i < options_nr; i++) {
+		opt = &longopt_all[options[i]];
+		longopt[i] = *opt;
+		if (opt->val != '\0') {
+			shortopt[j++] = (char) opt->val;
+			if (opt->has_arg)
+				shortopt[j++] = ':';
+		}
+	}
+	longopt[i] = longopt_all[OPT_NULL_ELEMENT];
+	shortopt[j] = '\0';
+
+	*_longopt = longopt;
+	*_shortopt = shortopt;
+}
+
+/**
+ * @brief Gets id for given CLI option which corresponds to value from longopt
+ * table.
+ *
+ * This is internal function of monitor service. The function maps given string
+ * to the value from longopt enum and returns id of matched element. Generic
+ * parameters allow to use this function for any CLI options-table which bases
+ * on option struct.
+ *
+ * @param[in]     optarg          String containing value given by user in CLI.
+ *
+ * @return integer id if successful, otherwise a -1.
+ */
+int get_option_id(const char *optarg)
+{
+	int i = 0;
+
+	while (longopt_all[i].name != NULL) {
+		if (strcmp(longopt_all[i].name, optarg) == 0)
+			return i;
+		i++;
+	}
+	return -1;
+}
+
+
+/**
+ * @brief Sets verbose variable to given level.
+ *
+ * This is internal function of monitor service. The function maps given level
+ * to the value from verbose_level enum and sets verbose value to ledmon
+ * configuration.
+ *
+ * @param[in]      log_level     required new log_level.
+ *
+ * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
+ */
+status_t set_verbose_level(int log_level)
+{
+	int new_verbose = -1;
+
+	switch (log_level) {
+	case OPT_ALL:
+		new_verbose = LOG_LEVEL_ALL;
+		break;
+	case OPT_DEBUG:
+		new_verbose = LOG_LEVEL_DEBUG;
+		break;
+	case OPT_ERROR:
+		new_verbose = LOG_LEVEL_ERROR;
+		break;
+	case OPT_INFO:
+		new_verbose = LOG_LEVEL_INFO;
+		break;
+	case OPT_QUIET:
+		new_verbose = LOG_LEVEL_QUIET;
+		break;
+	case OPT_WARNING:
+		new_verbose = LOG_LEVEL_WARNING;
+		break;
+	}
+	if (new_verbose != -1) {
+		conf.log_level = new_verbose;
+		return STATUS_SUCCESS;
+	}
+	return STATUS_CMDLINE_ERROR;
+}
+
+const char *ibpi2str(enum ibpi_pattern ibpi)
+{
+#ifdef _TEST_CONFIG
+	return NULL;
+#else
+	static char buf[20];
+	const char *ret;
+
+	if (ibpi >= 0 && ibpi < ibpi_pattern_count)
+		ret = ibpi_str[ibpi];
+	else
+		ret = NULL;
+
+	if (!ret) {
+		snprintf(buf, sizeof(buf), "(unknown: %u)", ibpi);
+		ret = buf;
+	}
+
+	return ret;
+#endif
+}
